@@ -18,8 +18,6 @@
 # Modified by Kai Willadsen for the Meld project
 # Copyright (C) 2013-2014 Kai Willadsen <kai.willadsen@gmail.com>
 
-from __future__ import print_function
-
 import distutils.cmd
 import distutils.command.build
 import distutils.command.build_py
@@ -156,8 +154,9 @@ class build_help(distutils.cmd.Command):
             path_help = os.path.join('share', 'help', lang, name)
             path_figures = os.path.join(path_help, 'figures')
             data_files.append((path_help, xml_files + mallard_files))
-            data_files.append(
-                (path_figures, glob.glob('%s/figures/*.png' % build_path)))
+            figures = glob.glob('%s/figures/*.png' % build_path)
+            if figures:
+                data_files.append((path_figures, figures))
 
         return data_files
 
@@ -186,7 +185,7 @@ class build_help(distutils.cmd.Command):
 class build_icons(distutils.cmd.Command):
 
     icon_dir = os.path.join("data", "icons")
-    target = "share/icons/"
+    target = "share/icons"
     frozen_target = "share/meld/icons"
 
     def initialize_options(self):
