@@ -3,13 +3,14 @@ import os
 import sys
 
 __package__ = "meld"
-__version__ = "3.16.3"
+__version__ = "3.17.0"
 
 # START; these paths are clobbered on install by meld.build_helpers
 DATADIR = os.path.join(sys.prefix, "share", "meld")
 LOCALEDIR = os.path.join(sys.prefix, "share", "locale")
 # END
 UNINSTALLED = False
+UNINSTALLED_SCHEMA = False
 
 # Installed from main script
 _ = lambda x: x
@@ -17,12 +18,13 @@ ngettext = lambda x, *args: x
 
 
 def frozen():
-    global DATADIR, LOCALEDIR
+    global DATADIR, LOCALEDIR, UNINSTALLED_SCHEMA
 
     melddir = os.path.dirname(sys.executable)
 
     DATADIR = os.path.join(melddir, "share", "meld")
     LOCALEDIR = os.path.join(melddir, "share", "mo")
+    UNINSTALLED_SCHEMA = True
 
     # This first bit should be unnecessary, but some things (GTK icon theme
     # location, GSettings schema location) don't fall back correctly.
@@ -32,13 +34,14 @@ def frozen():
 
 
 def uninstalled():
-    global DATADIR, LOCALEDIR, UNINSTALLED
+    global DATADIR, LOCALEDIR, UNINSTALLED, UNINSTALLED_SCHEMA
     melddir = os.path.abspath(os.path.join(
         os.path.dirname(os.path.realpath(__file__)), ".."))
 
     DATADIR = os.path.join(melddir, "data")
     LOCALEDIR = os.path.join(melddir, "build", "mo")
     UNINSTALLED = True
+    UNINSTALLED_SCHEMA = True
 
     # This first bit should be unnecessary, but some things (GTK icon theme
     # location, GSettings schema location) don't fall back correctly.
