@@ -1,5 +1,5 @@
 # Copyright (C) 2002-2009 Stephen Kennedy <stevek@gnome.org>
-# Copyright (C) 2009-2013 Kai Willadsen <kai.willadsen@gmail.com>
+# Copyright (C) 2009-2013, 2015 Kai Willadsen <kai.willadsen@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -121,9 +121,7 @@ class DiffMap(Gtk.DrawingArea):
         x0 = xpad
         x1 = width - 2 * xpad
 
-        # Hack to work around a cairo bug when calling create_similar
-        # https://bugs.freedesktop.org/show_bug.cgi?id=60519
-        if not (width and height):
+        if not (width > 0 and height > 0):
             return
 
         context.translate(0, y_start)
@@ -168,9 +166,9 @@ class DiffMap(Gtk.DrawingArea):
 
     def do_button_press_event(self, event):
         if event.button == 1:
-            y_start = self.get_allocation().y - self._scroll_y - self._y_offset
+            y_start = self._scroll_y + self._y_offset
             total_height = self._scroll_height - self._h_offset
-            fraction = (event.y + y_start) / total_height
+            fraction = (event.y - y_start) / total_height
 
             adj = self._scrolladj
             val = fraction * adj.get_upper() - adj.get_page_size() / 2
