@@ -21,6 +21,7 @@ from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
 
+import meld.ui.util
 from . import dirdiff
 from . import filediff
 from . import filemerge
@@ -32,10 +33,10 @@ from . import vcview
 from .ui import gnomeglade
 from .ui import notebooklabel
 
-from .util.compat import string_types
 from meld.conf import _
 from meld.recent import recent_comparisons
 from meld.settings import interface_settings, settings
+from meld.windowstate import SavedWindowState
 
 
 class MeldWindow(gnomeglade.Component):
@@ -46,13 +47,13 @@ class MeldWindow(gnomeglade.Component):
 
         actions = (
             ("FileMenu", None, _("_File")),
-            ("New", Gtk.STOCK_NEW, _("_New Comparison..."), "<control>N",
+            ("New", Gtk.STOCK_NEW, _("_New Comparison..."), "<Primary>N",
                 _("Start a new comparison"),
                 self.on_menu_file_new_activate),
             ("Save", Gtk.STOCK_SAVE, None, None,
                 _("Save the current file"),
                 self.on_menu_save_activate),
-            ("SaveAs", Gtk.STOCK_SAVE_AS, _("Save As..."), "<control><shift>S",
+            ("SaveAs", Gtk.STOCK_SAVE_AS, _("Save As..."), "<Primary><shift>S",
                 _("Save the current file with a different name"),
                 self.on_menu_save_as_activate),
             ("Close", Gtk.STOCK_CLOSE, None, None,
@@ -60,10 +61,10 @@ class MeldWindow(gnomeglade.Component):
                 self.on_menu_close_activate),
 
             ("EditMenu", None, _("_Edit")),
-            ("Undo", Gtk.STOCK_UNDO, None, "<control>Z",
+            ("Undo", Gtk.STOCK_UNDO, None, "<Primary>Z",
                 _("Undo the last action"),
                 self.on_menu_undo_activate),
-            ("Redo", Gtk.STOCK_REDO, None, "<control><shift>Z",
+            ("Redo", Gtk.STOCK_REDO, None, "<Primary><shift>Z",
                 _("Redo the last undone action"),
                 self.on_menu_redo_activate),
             ("Cut", Gtk.STOCK_CUT, None, None, _("Cut the selection"),
@@ -74,14 +75,14 @@ class MeldWindow(gnomeglade.Component):
                 self.on_menu_paste_activate),
             ("Find", Gtk.STOCK_FIND, _("Find..."), None, _("Search for text"),
                 self.on_menu_find_activate),
-            ("FindNext", None, _("Find Ne_xt"), "<control>G",
+            ("FindNext", None, _("Find Ne_xt"), "<Primary>G",
                 _("Search forwards for the same text"),
                 self.on_menu_find_next_activate),
-            ("FindPrevious", None, _("Find _Previous"), "<control><shift>G",
+            ("FindPrevious", None, _("Find _Previous"), "<Primary><shift>G",
                 _("Search backwards for the same text"),
                 self.on_menu_find_previous_activate),
             ("Replace", Gtk.STOCK_FIND_AND_REPLACE,
-                _("_Replace..."), "<control>H",
+                _("_Replace..."), "<Primary>H",
                 _("Find and replace text"),
                 self.on_menu_replace_activate),
 
@@ -104,25 +105,9 @@ class MeldWindow(gnomeglade.Component):
             ("Stop", Gtk.STOCK_STOP, None, "Escape",
                 _("Stop the current action"),
                 self.on_toolbar_stop_clicked),
-            ("Refresh", Gtk.STOCK_REFRESH, None, "<control>R",
+            ("Refresh", Gtk.STOCK_REFRESH, None, "<Primary>R",
                 _("Refresh the view"),
                 self.on_menu_refresh_activate),
-
-            ("TabMenu", None, _("_Tabs")),
-            ("PrevTab",   None, _("_Previous Tab"), "<Ctrl><Alt>Page_Up",
-                _("Activate previous tab"),
-                self.on_prev_tab),
-            ("NextTab",   None, _("_Next Tab"), "<Ctrl><Alt>Page_Down",
-                _("Activate next tab"),
-                self.on_next_tab),
-            ("MoveTabPrev", None,
-                _("Move Tab _Left"), "<Ctrl><Alt><Shift>Page_Up",
-                _("Move current tab to left"),
-                self.on_move_tab_prev),
-            ("MoveTabNext", None,
-                _("Move Tab _Right"), "<Ctrl><Alt><Shift>Page_Down",
-                _("Move current tab to right"),
-                self.on_move_tab_next),
         )
         toggleactions = (
             ("Fullscreen", None, _("Fullscreen"), "F11",
@@ -154,7 +139,7 @@ class MeldWindow(gnomeglade.Component):
         # Manually handle shells that don't show an application menu
         gtk_settings = Gtk.Settings.get_default()
         if not gtk_settings.props.gtk_shell_shows_app_menu:
-            from meldapp import app
+            from meld.meldapp import app
 
             def make_app_action(name):
                 def app_action(*args):
@@ -183,9 +168,6 @@ class MeldWindow(gnomeglade.Component):
             self.ui.add_ui_from_file(ui_file)
             self.widget.set_show_menubar(False)
 
-        self.tab_switch_actiongroup = None
-        self.tab_switch_merge_id = None
-
         for menuitem in ("Save", "Undo"):
             self.actiongroup.get_action(menuitem).props.is_important = True
         self.widget.add_accel_group(self.ui.get_accel_group())
@@ -204,9 +186,9 @@ class MeldWindow(gnomeglade.Component):
 
         # Add alternate keybindings for Prev/Next Change
         accels = self.ui.get_accel_group()
-        (keyval, mask) = Gtk.accelerator_parse("<Ctrl>D")
+        (keyval, mask) = Gtk.accelerator_parse("<Primary>D")
         accels.connect(keyval, mask, 0, self.on_menu_edit_down_activate)
-        (keyval, mask) = Gtk.accelerator_parse("<Ctrl>E")
+        (keyval, mask) = Gtk.accelerator_parse("<Primary>E")
         accels.connect(keyval, mask, 0, self.on_menu_edit_up_activate)
         (keyval, mask) = Gtk.accelerator_parse("F5")
         accels.connect(keyval, mask, 0, self.on_menu_refresh_activate)
@@ -224,6 +206,15 @@ class MeldWindow(gnomeglade.Component):
             Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
         self.toolbar_holder.pack_end(self.secondary_toolbar, False, True, 0)
 
+        # Manually handle GAction additions
+        actions = (
+            ("close", self.on_menu_close_activate, None),
+        )
+        for (name, callback, accel) in actions:
+            action = Gio.SimpleAction.new(name, None)
+            action.connect('activate', callback)
+            self.widget.add_action(action)
+
         toolbutton = Gtk.ToolItem()
         self.spinner = Gtk.Spinner()
         toolbutton.add(self.spinner)
@@ -239,14 +230,15 @@ class MeldWindow(gnomeglade.Component):
         self.widget.drag_dest_add_uri_targets()
         self.widget.connect("drag_data_received",
                             self.on_widget_drag_data_received)
+
+        self.window_state = SavedWindowState()
+        self.window_state.bind(self.widget)
+
+        self.should_close = False
         self.idle_hooked = 0
         self.scheduler = task.LifoScheduler()
         self.scheduler.connect("runnable", self.on_scheduler_runnable)
-        window_size = settings.get_value('window-size')
-        self.widget.set_default_size(window_size[0], window_size[1])
-        window_state = settings.get_string('window-state')
-        if window_state == 'maximized':
-            self.widget.maximize()
+
         self.ui.ensure_update()
         self.diff_handler = None
         self.undo_handlers = tuple()
@@ -257,6 +249,14 @@ class MeldWindow(gnomeglade.Component):
         rmenu = self.ui.get_widget('/Menubar/FileMenu/Recent').get_submenu()
         rmenu.connect("map", self._on_recentmenu_map)
 
+        try:
+            builder = meld.ui.util.get_builder("shortcuts.ui")
+            shortcut_window = builder.get_object("shortcuts-meld")
+            self.widget.set_help_overlay(shortcut_window)
+        except GLib.Error:
+            # GtkShortcutsWindow is new in GTK+ 3.20
+            pass
+
     def _on_recentmenu_map(self, recentmenu):
         for imagemenuitem in recentmenu.get_children():
             imagemenuitem.set_tooltip_text(imagemenuitem.get_label())
@@ -280,7 +280,7 @@ class MeldWindow(gnomeglade.Component):
 
     def on_idle(self):
         ret = self.scheduler.iteration()
-        if ret and isinstance(ret, string_types):
+        if ret and isinstance(ret, str):
             self.spinner.set_tooltip_text(ret)
 
         pending = self.scheduler.tasks_pending()
@@ -300,26 +300,26 @@ class MeldWindow(gnomeglade.Component):
             self.idle_hooked = GLib.idle_add(self.on_idle)
 
     def on_delete_event(self, *extra):
+        should_cancel = False
         # Delete pages from right-to-left.  This ensures that if a version
         # control page is open in the far left page, it will be closed last.
         for c in reversed(self.notebook.get_children()):
             page = c.pyobject
             self.notebook.set_current_page(self.notebook.page_num(page.widget))
-            response = self.try_remove_page(page, appquit=1)
+            response = page.on_delete_event()
             if response == Gtk.ResponseType.CANCEL:
-                return True
+                should_cancel = True
+
+        should_cancel = should_cancel or self.has_pages()
+        if should_cancel:
+            self.should_close = True
+        return should_cancel
 
     def has_pages(self):
         return self.notebook.get_n_pages() > 0
 
     def _update_page_action_sensitivity(self):
         current_page = self.notebook.get_current_page()
-        have_prev_tab = current_page > 0
-        have_next_tab = current_page < self.notebook.get_n_pages() - 1
-        self.actiongroup.get_action("PrevTab").set_sensitive(have_prev_tab)
-        self.actiongroup.get_action("NextTab").set_sensitive(have_next_tab)
-        self.actiongroup.get_action("MoveTabPrev").set_sensitive(have_prev_tab)
-        self.actiongroup.get_action("MoveTabNext").set_sensitive(have_next_tab)
 
         if current_page != -1:
             page = self.notebook.get_nth_page(current_page).pyobject
@@ -340,20 +340,23 @@ class MeldWindow(gnomeglade.Component):
                            "Replace"):
                 self.actiongroup.get_action(action).set_sensitive(is_filediff)
 
+    def handle_current_doc_switch(self, page):
+        if self.diff_handler is not None:
+            page.disconnect(self.diff_handler)
+        page.on_container_switch_out_event(self.ui)
+        if self.undo_handlers:
+            undoseq = page.undosequence
+            for handler in self.undo_handlers:
+                undoseq.disconnect(handler)
+            self.undo_handlers = tuple()
+
     def on_switch_page(self, notebook, page, which):
         oldidx = notebook.get_current_page()
         if oldidx >= 0:
             olddoc = notebook.get_nth_page(oldidx).pyobject
-            if self.diff_handler is not None:
-                olddoc.disconnect(self.diff_handler)
-            olddoc.on_container_switch_out_event(self.ui)
-            if self.undo_handlers:
-                undoseq = olddoc.undosequence
-                for handler in self.undo_handlers:
-                    undoseq.disconnect(handler)
-                self.undo_handlers = tuple()
+            self.handle_current_doc_switch(olddoc)
 
-        newdoc = notebook.get_nth_page(which).pyobject
+        newdoc = notebook.get_nth_page(which).pyobject if which >= 0 else None
         try:
             undoseq = newdoc.undosequence
             can_undo = undoseq.can_undo()
@@ -373,9 +376,13 @@ class MeldWindow(gnomeglade.Component):
         else:
             self.actiongroup.get_action("SaveAs").set_sensitive(True)
 
-        nbl = self.notebook.get_tab_label(newdoc.widget)
-        self.widget.set_title(nbl.get_label_text() + " - Meld")
-        newdoc.on_container_switch_in_event(self.ui)
+        if newdoc:
+            nbl = self.notebook.get_tab_label(newdoc.widget)
+            self.widget.set_title(nbl.get_label_text())
+            newdoc.on_container_switch_in_event(self.ui)
+        else:
+            self.widget.set_title("Meld")
+
         if isinstance(newdoc, melddoc.MeldDoc):
             self.diff_handler = newdoc.connect("next-diff-changed",
                                                self.on_next_diff_changed)
@@ -386,33 +393,12 @@ class MeldWindow(gnomeglade.Component):
 
     def after_switch_page(self, notebook, page, which):
         self._update_page_action_sensitivity()
-        actiongroup = self.tab_switch_actiongroup
-        if actiongroup:
-            action_name = "SwitchTab%d" % which
-            actiongroup.get_action(action_name).set_active(True)
 
     def after_page_reordered(self, notebook, page, page_num):
         self._update_page_action_sensitivity()
 
-    def on_notebook_label_changed(self, component, text, tooltip):
-        page = component.widget
-        nbl = self.notebook.get_tab_label(page)
-        nbl.set_label_text(text)
-        nbl.set_tooltip_text(tooltip)
-
-        # Only update the window title if the current page is active
-        if self.notebook.get_current_page() == self.notebook.page_num(page):
-            self.widget.set_title(text + " - Meld")
-        if isinstance(text, unicode):
-            text = text.encode('utf8')
-        self.notebook.child_set_property(page, "menu-label", text)
-
-        actiongroup = self.tab_switch_actiongroup
-        if actiongroup:
-            idx = self.notebook.page_num(page)
-            action_name = "SwitchTab%d" % idx
-            label = text.replace("_", "__")
-            actiongroup.get_action(action_name).set_label(label)
+    def on_page_label_changed(self, notebook, label_text):
+        self.widget.set_title(label_text)
 
     def on_can_undo(self, undosequence, can):
         self.actiongroup.get_action("Undo").set_sensitive(can)
@@ -424,17 +410,6 @@ class MeldWindow(gnomeglade.Component):
         self.actiongroup.get_action("PrevChange").set_sensitive(have_prev)
         self.actiongroup.get_action("NextChange").set_sensitive(have_next)
 
-    def on_configure_event(self, window, event):
-        state = event.window.get_state()
-        nosave = Gdk.WindowState.FULLSCREEN | Gdk.WindowState.MAXIMIZED
-        if not (state & nosave):
-            variant = GLib.Variant('(ii)', (event.width, event.height))
-            settings.set_value('window-size', variant)
-
-        maximised = state & Gdk.WindowState.MAXIMIZED
-        window_state = 'maximized' if maximised else 'normal'
-        settings.set_string('window-state', window_state)
-
     def on_menu_file_new_activate(self, menuitem):
         self.append_new_comparison()
 
@@ -458,7 +433,7 @@ class MeldWindow(gnomeglade.Component):
         i = self.notebook.get_current_page()
         if i >= 0:
             page = self.notebook.get_nth_page(i).pyobject
-            self.try_remove_page(page)
+            page.on_delete_event()
 
     def on_menu_undo_activate(self, *extra):
         self.current_doc().on_undo_activate()
@@ -522,90 +497,31 @@ class MeldWindow(gnomeglade.Component):
     def on_toolbar_stop_clicked(self, *args):
         self.current_doc().stop()
 
-    def on_prev_tab(self, *args):
-        self.notebook.prev_page()
+    def page_removed(self, page, status):
+        if hasattr(page, 'scheduler'):
+            self.scheduler.remove_scheduler(page.scheduler)
 
-    def on_next_tab(self, *args):
-        self.notebook.next_page()
+        page_num = self.notebook.page_num(page.widget)
 
-    def on_move_tab_prev(self, *args):
-        page_num = self.notebook.get_current_page()
-        child = self.notebook.get_nth_page(page_num)
-        page_num = page_num - 1 if page_num > 0 else 0
-        self.notebook.reorder_child(child, page_num)
+        if self.notebook.get_current_page() == page_num:
+            self.handle_current_doc_switch(page)
 
-    def on_move_tab_next(self, *args):
-        page_num = self.notebook.get_current_page()
-        child = self.notebook.get_nth_page(page_num)
-        self.notebook.reorder_child(child, page_num + 1)
+        self.notebook.remove_page(page_num)
+        # Normal switch-page handlers don't get run for removing the
+        # last page from a notebook.
+        if not self.has_pages():
+            self.on_switch_page(self.notebook, page, -1)
+            self._update_page_action_sensitivity()
+            if self.should_close:
+                cancelled = self.widget.emit(
+                    'delete-event', Gdk.Event.new(Gdk.EventType.DELETE))
+                if not cancelled:
+                    self.widget.emit('destroy')
 
-    def _update_notebook_menu(self, *args):
-        if self.tab_switch_merge_id:
-            self.ui.remove_ui(self.tab_switch_merge_id)
-            self.ui.remove_action_group(self.tab_switch_actiongroup)
-            self.ui.ensure_update()
-            self.tab_switch_merge_id = None
-            self.tab_switch_actiongroup = None
-
-        if not self.notebook.get_n_pages():
-            return
-
-        self.tab_switch_merge_id = self.ui.new_merge_id()
-        self.tab_switch_actiongroup = Gtk.ActionGroup(name="TabSwitchActions")
-        self.ui.insert_action_group(self.tab_switch_actiongroup)
-        group = None
-        current_page = self.notebook.get_current_page()
-        for i in range(self.notebook.get_n_pages()):
-            page = self.notebook.get_nth_page(i)
-            label = self.notebook.get_menu_label_text(page) or ""
-            label = label.replace("_", "__")
-            name = "SwitchTab%d" % i
-            tooltip = _("Switch to this tab")
-            action = Gtk.RadioAction(
-                name=name, label=label, tooltip=tooltip,
-                stock_id=None, value=i)
-            action.join_group(group)
-            group = action
-            action.set_active(current_page == i)
-
-            def current_tab_changed_cb(action, current):
-                if action == current:
-                    self.notebook.set_current_page(action.get_current_value())
-            action.connect("changed", current_tab_changed_cb)
-            if i < 10:
-                accel = "<Alt>%d" % ((i + 1) % 10)
-            else:
-                accel = None
-            self.tab_switch_actiongroup.add_action_with_accel(action, accel)
-            self.ui.add_ui(self.tab_switch_merge_id,
-                           "/Menubar/TabMenu/TabPlaceholder",
-                           name, name, Gtk.UIManagerItemType.MENUITEM, False)
-
-    def try_remove_page(self, page, appquit=0):
-        "See if a page will allow itself to be removed"
-        response = page.on_delete_event(appquit)
-        if response != Gtk.ResponseType.CANCEL:
-            if hasattr(page, 'scheduler'):
-                self.scheduler.remove_scheduler(page.scheduler)
-            page_num = self.notebook.page_num(page.widget)
-            assert page_num >= 0
-
-            # If the page we're removing is the current page, we need to
-            # disconnect and clear undo handlers, and trigger a switch out
-            if self.notebook.get_current_page() == page_num:
-                if self.diff_handler is not None:
-                    page.disconnect(self.diff_handler)
-                if self.undo_handlers:
-                    for handler in self.undo_handlers:
-                        page.undosequence.disconnect(handler)
-                self.undo_handlers = tuple()
-                page.on_container_switch_out_event(self.ui)
-
-            self.notebook.remove_page(page_num)
-            if self.notebook.get_n_pages() == 0:
-                self.widget.set_title("Meld")
-                self._update_page_action_sensitivity()
-        return response
+    def on_page_state_changed(self, page, old_state, new_state):
+        if self.should_close and old_state == melddoc.STATE_CLOSING:
+            # Cancel closing if one of our tabs does
+            self.should_close = False
 
     def on_file_changed(self, srcpage, filename):
         for c in self.notebook.get_children():
@@ -614,8 +530,8 @@ class MeldWindow(gnomeglade.Component):
                 page.on_file_changed(filename)
 
     def _append_page(self, page, icon):
-        nbl = notebooklabel.NotebookLabel(icon, "",
-                                          lambda b: self.try_remove_page(page))
+        nbl = notebooklabel.NotebookLabel(
+            icon, "", lambda b: page.on_delete_event())
         self.notebook.append_page(page.widget, nbl)
 
         # Change focus to the newly created page only if the user is on a
@@ -629,20 +545,21 @@ class MeldWindow(gnomeglade.Component):
         if hasattr(page, 'scheduler'):
             self.scheduler.add_scheduler(page.scheduler)
         if isinstance(page, melddoc.MeldDoc):
-            page.connect("label-changed", self.on_notebook_label_changed)
             page.connect("file-changed", self.on_file_changed)
             page.connect("create-diff", lambda obj, arg, kwargs:
                          self.append_diff(arg, **kwargs))
+            page.connect("state-changed", self.on_page_state_changed)
+        page.connect("close", self.page_removed)
 
         self.notebook.set_tab_reorderable(page.widget, True)
 
     def append_new_comparison(self):
         doc = newdifftab.NewDiffTab(self)
         self._append_page(doc, "document-new")
-        self.on_notebook_label_changed(doc, _("New comparison"), None)
+        self.notebook.on_label_changed(doc, _("New comparison"), None)
 
         def diff_created_cb(doc, newdoc):
-            self.try_remove_page(doc)
+            doc.on_delete_event()
             idx = self.notebook.page_num(newdoc.widget)
             self.notebook.set_current_page(idx)
 
