@@ -199,9 +199,9 @@ def colour_lookup_with_fallback(name, attribute):
 
     if not style_attr:
         import sys
-        print >> sys.stderr, _(
+        print(_(
             "Couldn't find colour scheme details for %s-%s; "
-            "this is a bad install") % (name, attribute)
+            "this is a bad install") % (name, attribute), file=sys.stderr)
         sys.exit(1)
 
     return parse_rgba(style_attr)
@@ -230,31 +230,6 @@ def gdk_to_cairo_color(color):
     return (color.red / 65535., color.green / 65535., color.blue / 65535.)
 
 
-def fallback_decode(bytes, encodings, lossy=False):
-    """Try and decode bytes according to multiple encodings
-
-    Generally, this should be used for best-effort decoding, when the
-    desired behaviour is "probably this, or UTF-8".
-
-    If lossy is True, then decode errors will be replaced. This may be
-    reasonable when the string is for display only.
-    """
-    if isinstance(bytes, unicode):
-        return bytes
-
-    for encoding in encodings:
-        try:
-            return bytes.decode(encoding)
-        except UnicodeDecodeError:
-            pass
-
-    if lossy:
-        return bytes.decode(encoding, errors='replace')
-
-    raise ValueError(
-        "Couldn't decode %r as one of %r" % (bytes, encodings))
-
-
 def all_same(lst):
     """Return True if all elements of the list are equal"""
     return not lst or lst.count(lst[0]) == len(lst)
@@ -307,10 +282,10 @@ def read_pipe_iter(command, workdir, errorstream, yield_interval=0.1):
                                   self.proc.wait())
 
         def __call__(self):
-            self.proc = subprocess.Popen(command, cwd=workdir,
-                                         stdin=subprocess.PIPE,
-                                         stdout=subprocess.PIPE,
-                                         stderr=subprocess.PIPE)
+            self.proc = subprocess.Popen(
+                command, cwd=workdir, stdin=subprocess.PIPE,
+                stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                universal_newlines=True)
             self.proc.stdin.close()
             childout, childerr = self.proc.stdout, self.proc.stderr
             bits = []
@@ -507,10 +482,14 @@ def apply_text_filters(txt, regexes, apply_fn=None):
     """Apply text filters
 
     Text filters "regexes", resolved as regular expressions are applied
-    to "txt".
+    to "txt". "txt" may be either strings or bytes, but the supplied
+    regexes must match the type.
 
     "apply_fn" is a callable run for each filtered interval
     """
+    empty_string = b"" if isinstance(txt, bytes) else ""
+    newline = b"\n" if isinstance(txt, bytes) else "\n"
+
     filter_ranges = []
     for r in regexes:
         for match in r.finditer(txt):
@@ -538,8 +517,8 @@ def apply_text_filters(txt, regexes, apply_fn=None):
     offset = 0
     result_txts = []
     for (start, end) in filter_ranges:
-        assert txt[start:end].count("\n") == 0
+        assert txt[start:end].count(newline) == 0
         result_txts.append(txt[offset:start])
         offset = end
     result_txts.append(txt[offset:])
-    return "".join(result_txts)
+    return empty_string.join(result_txts)
